﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Dynamic;

namespace customDynamicApp
{
    public sealed class MyDynamicObject : DynamicObject
    {
        #region 私有字段
        Dictionary<string, object> dic = null;
        #endregion

        #region 构造方法
        public MyDynamicObject()
        {
            dic = new Dictionary<string, object>();
        }
        #endregion

        #region 重写的方法
        /// <summary>
        /// 设置成员的值，如属性。
        /// </summary>
        /// <param name="binder"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public override bool TrySetMember(SetMemberBinder binder, object value)
        {
            // binder.Name属性表示设置的成员名字，
            // 比如动态对象变量k，k.Age = 22，则binder.Name为"Age"，
            // 此处把它转化为小写字母，作为字典的Key来存储
            dic[binder.Name.ToLower()] = value;
            /*
            如果操作成功，返回true；
            如果操作不成功，返回false。
            此处直接操作字典数据，基本不会发生错误，
            所以直接返回true。
            */
            return true;
        }

        /// <summary>
        /// 获取成员的值，如属性等。
        /// </summary>
        /// <param name="binder"></param>
        /// <param name="result"></param>
        /// <returns></returns>
        public override bool TryGetMember(GetMemberBinder binder, out object result)
        {
            string key = binder.Name.ToLower();
            if (dic.ContainsKey(key))
            {
                result = dic[key];
                return true;
            }
            result = null;
            return false;
        }

        /// <summary>
        /// 方法调用实现
        /// </summary>
        /// <param name="binder"></param>
        /// <param name="args"></param>
        /// <param name="result"></param>
        /// <returns></returns>
        public override bool TryInvokeMember(InvokeMemberBinder binder, object[] args, out object result)
        {
            result = 0d;
            // 如果参数个数不是1个，则调用失败
            if (args.Length != 1) return false;
            // 如果参数类型不能隐式转化为double，则调用失败
            object a = args[0];
            if (a == null) return false;
            if ((typeof(double).IsAssignableFrom(a.GetType())) == false) return false;
            // 判断方法名称
            string methodName = binder.Name.ToLower();
            double num = Convert.ToDouble(a);
            if (methodName == "m2")
            {
                result = Math.Pow(num, 2d); //二次方
            }
            else if (methodName == "m3")
            {
                result = Math.Pow(num, 3d); //三次方
            }
            return true;
        }
        #endregion
    }
}
